#include <stdio.h>
#include "ruby.h"
#include "spglib.h"

VALUE Getspg = Qnil;
void Init_getspg(void);
VALUE method_getspg(VALUE self, VALUE r_size, VALUE r_lattice, VALUE r_position, VALUE r_types, VALUE r_symprec);
VALUE method_getspg_long(VALUE self, VALUE r_size, VALUE r_lattice, VALUE r_position, VALUE r_types, VALUE r_symprec);
VALUE method_get_operations(VALUE self, VALUE r_size, VALUE r_lattice, VALUE r_position, VALUE r_types, VALUE r_symprec);

void Init_getspg(void)
{
  Getspg = rb_define_module("Getspg");
  rb_define_method(Getspg, "getspg", method_getspg, 5);
  rb_define_method(Getspg, "getspg_long", method_getspg_long, 5);
  rb_define_method(Getspg, "get_operations", method_get_operations, 5);
}

VALUE method_getspg(VALUE self, VALUE r_size, VALUE r_lattice, VALUE r_position, VALUE r_types, VALUE r_symprec)
{
  int i, j, size, spgroup;
  double symprec, lattice[3][3];
  VALUE array;
     
  size = NUM2INT(r_size);
     
  double position[size*4][3];
  int types[size*4];
  char symbol[21];
  char output[21];

  symprec = NUM2DBL(r_symprec);

  for (i=0; i<size; i++)
    for (j=0; j<3; j++) {
      position[i][j] =
	NUM2DBL(rb_ary_entry(rb_ary_entry(r_position, i), j));
      types[i] = NUM2DBL(rb_ary_entry(r_types, i));
    }

  for (i=0; i<3; i++)
    for (j=0; j<3; j++)
      lattice[i][j] =
	NUM2DBL(rb_ary_entry(rb_ary_entry(r_lattice, i), j));

  /* Space group */
  spgroup = spg_get_international( symbol,
				   lattice,
				   position,
				   types,
				   size,
				   symprec);
  sprintf(output, "%s", symbol);

  array = rb_ary_new();
  rb_ary_push(array, rb_str_new2(output));
  rb_ary_push(array, INT2NUM(spgroup));

  return array;
}

VALUE method_getspg_long(VALUE self, VALUE r_size, VALUE r_lattice, VALUE r_position, VALUE r_types, VALUE r_symprec)
{
  int i, j, size, spgroup, num_atom_bravais ;
  double symprec, lattice[3][3];
  /* , bravais_lattice[3][3]; */
  VALUE vector, array, r_brv_lattice, r_brv_positions, r_brv_types;
     
  size = NUM2INT(r_size);
     
  double position[size*4][3];
  int types[size*4];
  char symbol[21];
  char output[21];

  symprec = NUM2DBL(r_symprec);

  for (i=0; i<size; i++)
    for (j=0; j<3; j++) {
      position[i][j] =
	NUM2DBL(rb_ary_entry(rb_ary_entry(r_position, i), j));
      types[i] = NUM2DBL(rb_ary_entry(r_types, i));
    }

  for (i=0; i<3; i++)
    for (j=0; j<3; j++)
      lattice[i][j] =
	NUM2DBL(rb_ary_entry(rb_ary_entry(r_lattice, i), j));

  /* Space group */
  spgroup = spg_get_international( symbol,
				   lattice,
				   position,
				   types,
				   size,
				   symprec);
  sprintf(output, "%s", symbol);

  num_atom_bravais = spg_refine_cell( lattice,
  				      position,
  				      types,
  				      size,
  				      symprec);

  r_brv_lattice = rb_ary_new();
  for (i=0; i<3; i++) {
    vector = rb_ary_new();
    for (j=0; j<3; j++) {
      rb_ary_push(vector, rb_float_new(lattice[i][j]) );
    }
    rb_ary_push(r_brv_lattice, vector);
  }

  r_brv_positions = rb_ary_new();
  r_brv_types = rb_ary_new();
  for (i=0; i<num_atom_bravais; i++) {
    vector = rb_ary_new();
    for (j=0; j<3; j++) {
      rb_ary_push(vector, rb_float_new(position[i][j]) );
    }
    rb_ary_push(r_brv_positions, vector);
    rb_ary_push(r_brv_types, INT2NUM(types[i]));
  }

  array = rb_ary_new();
  rb_ary_push(array, rb_str_new2(output));
  rb_ary_push(array, r_brv_lattice);
  rb_ary_push(array, r_brv_positions);
  rb_ary_push(array, r_brv_types);
  rb_ary_push(array, INT2NUM(spgroup));

  return array;
}

VALUE method_get_operations(VALUE self, VALUE r_size, VALUE r_lattice, VALUE r_position, VALUE r_types, VALUE r_symprec)
{
  int i, j, k, num_atom, num_sym;
  double symprec, lattice[3][3];
  /* , bravais_lattice[3][3]; */
  VALUE matrix, matrix_row, vector, array, r_rotation, r_translation;
     
  num_atom = NUM2INT(r_size);
     
  double position[num_atom*4][3];
  int types[num_atom*4];
  int rotation[num_atom*192][3][3];
  double translation[num_atom*192][3];

  symprec = NUM2DBL(r_symprec);

  for (i=0; i<num_atom; i++)
    for (j=0; j<3; j++) {
      position[i][j] =
	NUM2DBL(rb_ary_entry(rb_ary_entry(r_position, i), j));
      types[i] = NUM2DBL(rb_ary_entry(r_types, i));
    }

  for (i=0; i<3; i++)
    for (j=0; j<3; j++)
      lattice[i][j] =
	NUM2DBL(rb_ary_entry(rb_ary_entry(r_lattice, i), j));

  num_sym = spg_get_symmetry( rotation,
			      translation,
			      num_atom*192,
			      lattice,
			      position,
			      types,
			      num_atom,
			      symprec );

  
  r_rotation = rb_ary_new();
  r_translation = rb_ary_new();

  for ( i = 0; i < num_sym; i++ ) {
    vector = rb_ary_new();
    matrix = rb_ary_new();
    for ( j = 0; j < 3 ; j++ ) {
      rb_ary_push( vector, rb_float_new( translation[i][j] ) );
      matrix_row = rb_ary_new();
      for ( k = 0; k < 3; k++ ) {
	rb_ary_push( matrix_row, INT2NUM( rotation[i][j][k] ) );
      }
      rb_ary_push( matrix, matrix_row );
    }
    rb_ary_push( r_rotation, matrix );
    rb_ary_push( r_translation, vector );
  }

  array = rb_ary_new();
  rb_ary_push(array, r_rotation);
  rb_ary_push(array, r_translation);

  return array;
}
